 /*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Widget;

import com.lombardisoftware.gwtcommon.client.clientObjects.AbstractClientObject;
import com.lombardisoftware.gwtcommon.client.clientObjects.Filter;
import com.lombardisoftware.gwtcommon.client.clientObjects.Sorter;
import com.lombardisoftware.gwtcommon.client.clientObjects.ComplexResult;
import com.lombardisoftware.gwtcommon.client.help.HelpTopicProvider;
import com.lombardisoftware.gwtcommon.client.help.HelpWidgetFactory;
import com.lombardisoftware.gwtcommon.client.Paginator.PageListener;

 /**
 * The base class for Repository panel with a list of items like Process App list or Server list.
 *
 * @author jsugg
 *
 */
public abstract class AbstractListPanel extends AbstractRepositoryPanel implements HelpTopicProvider {

    protected UnorderedListPanel filterPanel = null;
    protected ListBox sortMenu = null;
    protected FlowPanel links = new FlowPanel();
    protected Grid grid = null;

    protected List<SortItem> available_sorts = new ArrayList<SortItem>();
    protected List<FilterItem> available_filters = new ArrayList<FilterItem>();
   
    protected String selectedSort = null;
    protected Filter selectedFilter = Filter.ALL;
    private ScrollPanel gridScroll;
    private Paginator paginator;


    public AbstractListPanel() {
        super();
        paginator = new Paginator();
    }

    /**
     * Retrieve the list of objects to be displayed in the link panel. Sorts and filters should be applied to
     * this list.
     * @param callback    The callback object to be invoked when the data has been retrieved. onSuccess will be called back with a list of AbstractClientObjects.
     * @return    sorted, filtered list of objects to be displayed
     */
    protected abstract void getLinkObjects(AsyncCallback<?> callback); 

    /**
     * Build a link panel for the specified object
     * @param theObject    the object to be displayed
     * @return             the widget displaying the object
     */
    protected abstract Widget getLinkPanel(AbstractClientObject theObject); 

    /**
     * Display a new object in the getLinkPanel-generated widget object
     * 
     * @param theObject  The new object to display
     * @param widget  The widget generated from getLinkPanel that should display the new object
     */
    protected abstract void displayObject(AbstractClientObject theObject, Widget widget);
    
    public Widget getInformationWidget() {
        return HelpWidgetFactory.createInformationWidget(this);
    }

    public Widget getInformationLinkWidget(){
        return HelpWidgetFactory.createInformationLinkWidget(this);
    }

    public RefreshablePanel getContentsWidget() {
        
        // if we don't clear the links here we get multiple copies of the grid on subsequent refreshes.
        if (links != null)
            links.clear();
        
        ContentContainerPanel parent = new ContentContainerPanel();
        CommonUtils.addAutomationId(parent, "gwtcommon.client.contentContainer.panel_0");

        FlowPanel header = new FlowPanel();
        header.setStyleName("twSortFilterHeader");
        parent.add(header);

        FlowPanel pipe = new FlowPanel();
        CommonUtils.addAutomationId(pipe, "gwtcommon.client.contentContainer.pipe_0");
        pipe.getElement().setId("pipe");
        header.add(pipe);

        filterPanel = new UnorderedListPanel();
        CommonUtils.addAutomationId(filterPanel, "gwtcommon.client.contentContainer.unorderedListPanel_0");
        rebuildFilterPanel();
        pipe.add(filterPanel);

        HeadingPanel h1 = new HeadingPanel();
        CommonUtils.addAutomationId(h1, "gwtcommon.client.contentContainer.headingPanel_0");
        header.add(h1);

        if (available_sorts.size() > 0) {
            h1.addText(getSortLabel());
            sortMenu = new ListBox();
            CommonUtils.addAutomationId(sortMenu, "gwtcommon.client.contentContainer.listBox_0");
            rebuildSortMenu(sortMenu);
            h1.add(sortMenu);
        }

        Widget additionalHeaderContent = getAdditionalHeaderContent();
        if(additionalHeaderContent != null) {
            header.add(additionalHeaderContent);
        }

        links.getElement().setAttribute("id", "links");
        parent.add(links);

        paginator.addPageListener(new PageListener() {
            public void firePageChanged(int newPage) {
                rebuildLinks();
            }
        });

        initializeGrid();
        // make asynchronous call to build out the contents of the link panel
        // note: there's no callback passed in because we don't care when it
        //       finishes.
        rebuildLinks();
        
        parent.add(paginator.getWidget());
        
        return parent;
     }

    protected Paginator getPaginator() {
        return paginator;
    }
    /**
     * Create any additional content to go into the header (to the left of the sort list and filter pipe).
     *
     * @return The additional header content, or <tt>null</tt> if nothing
     */
    protected Widget getAdditionalHeaderContent() {
        return null;
    }

    protected void initializeGrid() {
        grid = new Grid(1, 1);
        grid.setCellPadding(0);
        grid.setCellSpacing(0);
        grid.setStyleName("linkContainer");
        gridScroll = new ScrollPanel(grid);
        CommonUtils.addAutomationId(gridScroll, "gwtcommon.client.contentContainer.scrollPanel_0");
        gridScroll.getElement().getStyle().setProperty("overflow", "");
        gridScroll.getElement().getStyle().setProperty("overflowX", "hidden");
        gridScroll.getElement().getStyle().setProperty("overflowY", "auto");
        gridScroll.getElement().getStyle().setProperty("width", "100%");
        grid.setWidth("100%");
        links.add(gridScroll);
    }
    
    protected String getSortLabel() {
        return CommonUtils.getCommonMessages().sortSelectorLabel();
    }

    protected void rebuildSortMenu(final ListBox menu) {
        menu.setName("jumpMenu");
        // build the menu based on the sorts specified by the concrete sub-class
        for (int i = 0; i < available_sorts.size(); i++) {
            SortItem sort = (SortItem) available_sorts.get(i);
            menu.addItem(sort.displayName, sort.id);
        }
        
        // when a selection changes indicate it by setting the selected_sort field and rebuilding.
        menu.addChangeListener(new ChangeListener() {
            public void onChange(Widget sender) {
                doSortAction(menu);
            }
        });
        menu.getElement().setAttribute("id", "jumpmenu");
    }

    protected void doSortAction(ListBox menu) {
        selectedSort = menu.getValue(menu.getSelectedIndex());
        rebuildLinks();
    }

    protected void rebuildFilterPanel() {
        if (filterPanel.getWidgetCount() > 0) {
            filterPanel.clear();
        }

        for (FilterItem available_filter : available_filters) {
            filterPanel.addListItem(buildFilterItem(available_filter));
        }
    }

    protected Widget buildFilterItem(final FilterItem filterItem) {
        FlowPanel panel = null;
        // if there's an image for this filter then we need to build a panel for it -- the rest after
        // this should proceed normally
        if (filterItem.style != null) {
            panel = new FlowPanel();
            CommonUtils.addAutomationId(panel, "gwtcommon.client.contentContainer.panel_1");
            panel.addStyleName("inline");
            panel.addStyleName(filterItem.style);
        }
        
        Widget item;
        if (!selectedFilter.equals(filterItem.id)) {
            item = new Hyperlink(filterItem.displayName, filterItem.id.toString());
            CommonUtils.addAutomationId(item, "gwtcommon.client.contentContainer.hyperlink_1");
            ((Hyperlink) item).addClickListener(new ClickListener() {
                public void onClick(Widget sender) {
                    doFilterAction(filterItem);
                }
            });
        } else {
            item = new Label(filterItem.displayName);
            CommonUtils.addAutomationId(item, "gwtcommon.client.contentContainer.label_0");
            item.setStyleName("bold");
        }

        // final details if there was an image specified
        if (panel != null) {
            item.addStyleName("inline");
            panel.add(item);
            item = panel;
        }
        
        return item;
    }

    protected void doFilterAction(FilterItem filterItem) {
        selectedFilter = filterItem.id;
        rebuildFilterPanel();
        rebuildLinks();
    }

    /**
     * Refresh the list from the server and rebuild the list display with the new data.
     */
    protected void rebuildLinks() {
        // retrieve the ordered list of objects with correct filters and sorts applied
        getLinkObjects(new BuildGridCallback());
    }

    
    /**
     * Utility class to represent displayed sorts.
     * @author jsugg
     */
    protected static class SortItem {
        public String displayName = "";
        public String id = Sorter.NONE_TAG;
        
        public SortItem(String displayName, String id) {
            this.displayName = displayName;
            this.id = id;
        }
    }
    
    /**
     * Utility class to represent displayed filters.
     * @author jsugg
     *
     */
    protected static class FilterItem {
        public String displayName = "";
        public Filter id = Filter.ALL;
        public String style = null;
        
        public FilterItem(String displayName, String style, Filter id) {
            this.displayName = displayName;
            this.style = style;
            this.id = id;
        }
    }

    
    /**
     * Utility callback class to build out the visual list of objects.
     * @author jsugg
     */
    public class BuildGridCallback implements AsyncCallback{

        public void onFailure(Throwable caught) {
            //CommonUtils.showException(caught, "Error building grid");
            //Window.alert("Unable to build grid because of exception \n" + caught);
        }

        public void onSuccess(Object result) {

            // NOTE: this method assumes the links member of the AbstractListPanel has been initialized!
            List objects;
            if (result instanceof ComplexResult) {
                objects = ((ComplexResult)result).getList();
            } else {
                objects = (List) result;
            }

            // make sure the grid matches the size of our data
            if (grid == null) {
                initializeGrid();
            }
            grid.resizeRows(objects.size());
            
            for (int i = 0; i < objects.size(); i++) {
                Widget panel = grid.getWidget(i, 0);

                if (panel != null) {
                    displayObject((AbstractClientObject) objects.get(i), panel);
                } else {
                    // build a new panel
                    panel = getLinkPanel((AbstractClientObject) objects.get(i));
                    grid.setWidget(i, 0, panel);
                }

                // Indexes are counting from 0 but rows from 1 so first row (i == 0) is odd.
                if (i % 2 == 0) {
                    grid.getRowFormatter().addStyleName(i, "odd");
                } else {
                    grid.getRowFormatter().addStyleName(i, "even");
                }
            }

            updateDimensionsDeferred();
        }
    }
    
    protected class ContentContainerPanel extends RefreshablePanel {

        public void refresh() {
            rebuildLinks();
        }
        
    }

    public void updateDimensions(int clientWidth, int clientHeight) {
        super.updateDimensions(clientWidth, clientHeight);

        if(gridScroll != null) {
            int gridScrollHeight = clientHeight - gridScroll.getAbsoluteTop() - 35;
            gridScroll.setHeight(gridScrollHeight + "px");
        }
    }
}
