/*
 * Copyright (C) 2007 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ToggleButton;
import com.google.gwt.user.client.ui.Widget;
import com.lombardisoftware.gwtcommon.client.clientObjects.AbstractClientObject;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPermission;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;

/**
 * @author Bryan Campbell
 */
public abstract class AbstractPermissionRow<T extends AbstractClientObject> extends PrincipalRow {

    private T object;

    private final ToggleButton read = new ToggleButton(CommonUtils.COMMON_IMAGES.itemUnchecked().createImage(), CommonUtils.COMMON_IMAGES.itemChecked().createImage());
    private final ToggleButton write = new ToggleButton(CommonUtils.COMMON_IMAGES.itemUnchecked().createImage(), CommonUtils.COMMON_IMAGES.itemChecked().createImage());
    private final ToggleButton admin = new ToggleButton(CommonUtils.COMMON_IMAGES.itemUnchecked().createImage(), CommonUtils.COMMON_IMAGES.itemChecked().createImage());

    /**
     * Creates a new ProcessRow for the given process.
     */
    public AbstractPermissionRow(ClientPrincipal principal, T object) {
        super(principal, true);

        this.object = object;

        read.setStyleName("rowItemButton");
        read.setEnabled(false);
        read.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                if(write.isDown()) {
                    write.setDown(false);
                }
                if(admin.isDown()) {
                    admin.setDown(false);
                }

                if(read.isDown()) {
                    doChangePrincipalPermissionLevel(ClientPermission.READ);
                }
            }
        });

        write.setStyleName("rowItemButton");
        write.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                if(!read.isDown()) {
                    read.setDown(true);
                }
                if(admin.isDown()) {
                    admin.setDown(false);
                }

                if(write.isDown()) {
                    doChangePrincipalPermissionLevel(ClientPermission.WRITE);
                }
                else{
                    doChangePrincipalPermissionLevel(ClientPermission.READ);
                }

            }
        });

        admin.setStyleName("rowItemButton");
        admin.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                if(!read.isDown()) {
                    read.setDown(true);
                }
                if(!write.isDown()) {
                    write.setDown(true);
                }

                if(admin.isDown()) {
                    doChangePrincipalPermissionLevel(ClientPermission.ADMINISTRATION);
                }
                else{
                    doChangePrincipalPermissionLevel(ClientPermission.WRITE);
                }
            }
        });

        // Set the check boxes based on the permissions
        for(ClientPermission p : principal.getPermissions()) {
            switch(p) {
                case READ:
                    read.setDown(true);
                    break;
                case WRITE:
                    write.setDown(true);
                    break;
                case ADMINISTRATION:
                    admin.setDown(true);
                    break;
            }
        }

        // Add the checkboxes to the panel
        this.insert(admin, 0);
        this.insert(write, 0);
        this.insert(read, 0);
    }

    public T getObject() {
        return object;
    }

    @Override
    protected abstract void doRemoveRow();

    protected abstract void doChangePrincipalPermissionLevel(ClientPermission permission);

    public static class Header extends PrincipalRow.Header {
        public Header() {
            super();

            Label readHeaderLabel = new Label("Read");
            readHeaderLabel.setStyleName("rowColumnHeader");

            Label writeHeaderlabel = new Label("Write");
            writeHeaderlabel.setStyleName("rowColumnHeader");

            Label adminHeaderLabel = new Label("Admin");
            adminHeaderLabel.setStyleName("rowColumnHeader");

            insert(adminHeaderLabel, 0);
            insert(writeHeaderlabel, 0);
            insert(readHeaderLabel, 0);
        }
    }

}
