package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;

/**
 * @author zroadhouse
 *
 */
public abstract class AbstractPrincipalPopup extends AbstractRepositoryPopup {

    protected static final int NAME_FIELD_WIDTH = 40;
    protected static final int DESCRIPTION_FIELD_WIDTH = NAME_FIELD_WIDTH;
    private TextBox nameText = new TextBox();
    
    /**
     * @param parent
     */
    public AbstractPrincipalPopup(Widget parent) {
        super(parent);
    }

    protected List<LibraryButton> getButtons() {
        List<LibraryButton> buttons = new ArrayList<LibraryButton>();
        LibraryButton save = new LibraryButton(CommonUtils.getCommonMessages().saveButtonLabel());
        CommonUtils.addAutomationId(save, "gwtcommon.client.abstractSnapshotPopup.libraryButton_0");
        save.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                submit();
            }
        });
        buttons.add(save);
        return buttons;
    }

    protected abstract void submit();
    
    protected Widget getContents() {

        FlowPanel contents = new FlowPanel();
        CommonUtils.addAutomationId(contents, "gwtcommon.client.abstractPrincipalPopup.flowPanel_1");
        
        Label nameLabel = new Label(CommonUtils.getCommonMessages().nameFieldLabel());
        contents.add(nameLabel);
        nameText = new TextBox();
        CommonUtils.addAutomationId(nameText, "gwtcommon.client.abstractPrincipalPopup.TextBox_0");

        nameText.setVisibleLength(NAME_FIELD_WIDTH);
        nameText.setText(getPrincipal().getName());
        nameText.addChangeListener(new ChangeListener() {
            public void onChange(Widget sender){
                String name = nameText.getText();
                // TODO: for editing an object we should be making a clone of the original
                getPrincipal().setName(name);
            }
        });
        contents.add(nameText);

        Label descriptionLabel = new Label(CommonUtils.getCommonMessages().descriptionFieldLabel());
        CommonUtils.addAutomationId(descriptionLabel, "gwtcommon.client.abstractPrincipalPopup.dscLabel_0");

        contents.add(descriptionLabel);
        final TextArea descriptionText = new TextArea();
        CommonUtils.addAutomationId(descriptionText, "gwtcommon.client.abstractPrincipalPopup.textArea_0");

        descriptionText.setVisibleLines(3);
        descriptionText.setCharacterWidth(DESCRIPTION_FIELD_WIDTH);
        descriptionText.setText(getPrincipal().getDescription());
        descriptionText.addChangeListener(new ChangeListener() {
            public void onChange(Widget sender){
                String description = descriptionText.getText();
                getPrincipal().setDescription(description);
            }
        });
        contents.add(descriptionText);

        return contents;
    }

    protected abstract ClientPrincipal getPrincipal();
    
    protected void focusOnDefault() {
        nameText.setFocus(true);
    }
    
    protected void onEnter() {
        submit();
    }
}
