/**
 * 
 */
package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.DOM;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;
import com.lombardisoftware.gwtcommon.client.clientObjects.ComplexResult;

/**
 * @author bcampbell
 *
 */
public abstract class AddUsersAndGroupsPopup extends AbstractRepositoryPopup {
    private static final String STYLE_NAME = "addUsersAndGroupsPopup";
    protected final UsersListWidget userList = new UsersListWidget();
    private final TextBox searchBox = new TextBox();
    Label searchInstructions = new Label();

    protected AsyncCallback<ComplexResult<ClientPrincipal>> populateUserListCallback = new AsyncCallback<ComplexResult<ClientPrincipal>>(){
        public void onFailure(Throwable caught) {
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(ComplexResult<ClientPrincipal> result) {
            userList.refreshUsers(result.getList());
        }
    };
    
    protected AsyncCallback submitCallback = new AsyncCallback() {
        public void onFailure(Throwable caught) {
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(Object result) {
            close();
        }
    };
        

    protected int userListLimitFromExternalSecurityProvider = -1;

    protected AsyncCallback<Integer> changeLabelTextCallback = new AsyncCallback<Integer>() {
        public void onFailure(Throwable caught) {
            ExceptionPopup.showException(caught);
        }

        public void onSuccess(Integer result) {
            userListLimitFromExternalSecurityProvider = result.intValue();
            searchInstructions.setText(CommonUtils.getCommonMessages().searchBoxInstructions(getOptionalLimit(), getExtraSearchBoxInstructions()));
        }
    };

    public AddUsersAndGroupsPopup(Widget parent) {
        super(parent);
        rebuildPanel();
    }

    protected List<LibraryButton> getButtons() {
        List<LibraryButton> buttons = new ArrayList<LibraryButton>();
        LibraryButton save = new LibraryButton(CommonUtils.getCommonMessages().addSelectedButtonName());
        CommonUtils.addAutomationId(save, "gwtcommon.client.addUsersAndGroupsPopup.libraryButton_0");
        save.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                submit();
            }
        });
        buttons.add(save);
        return buttons;
    }

    protected Widget getContents() {
        FlowPanel contents = new FlowPanel();
        CommonUtils.addAutomationId(contents, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_0");
        contents.setStyleName(STYLE_NAME);
        
        FlowPanel searchBoxLabel = new FlowPanel();
        CommonUtils.addAutomationId(searchBoxLabel, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_1");
        searchBoxLabel.add(new Label(CommonUtils.getCommonMessages().searchForName()));
        contents.add(searchBoxLabel);


        CommonUtils.addAutomationId(searchBox, "gwtcommon.client.addUsersAndGroupsPopup.textBox_0");
        searchBox.setWidth("40%");
        searchBox.addKeyboardListener(new KeyboardListener() {
            public void onKeyDown(Widget sender, char keyCode, int modifiers) {
            }

            public void onKeyPress(Widget sender, char keyCode, int modifiers) {
            }

            public void onKeyUp(Widget sender, char keyCode, int modifiers) {
                if(searchBox.getText().length() >= 3) {
                    loadUsers(searchBox.getText());
                }
            }
        });
        contents.add(searchBox);

        setLabelText();
        CommonUtils.addAutomationId(searchInstructions, "gwtcommon.client.addUsersAndGroupsPopup.label_0");
        searchInstructions.setStyleName("searchInstructions");
        searchInstructions.addStyleName("inline");
        contents.add(searchInstructions);

        FlowPanel resultsBoxLabel = new FlowPanel();
        CommonUtils.addAutomationId(resultsBoxLabel, "gwtcommon.client.addUsersAndGroupsPopup.flowPanel_2");
        resultsBoxLabel.add(new Label(CommonUtils.getCommonMessages().results()));
        contents.add(resultsBoxLabel);

        contents.add(userList);
        return contents;
    }

    protected void setLabelText() {
        searchInstructions.setText(CommonUtils.getCommonMessages().searchBoxInstructions(getOptionalLimit(), getExtraSearchBoxInstructions()));
    }

    protected String getOptionalLimit() {
        return "";
    }

    protected String getExtraSearchBoxInstructions() {
        return "";
    }
    
    protected String getTitleText() {
        return CommonUtils.getCommonMessages().addUserPopupTitle();
    }

    protected abstract void loadUsers(String prefix);

    protected abstract void submit();

    protected void onEnterKey() {
        super.onEnterKey();
        submit();
    }

    @Override
    protected void focusOnDefault() {
        super.focusOnDefault();
        searchBox.setFocus(true);
    }

    public List<ClientPrincipal> getSelectedPrincipals() {
        return userList.getSelectedPrincipals();
    }

    protected class UsersListWidget extends ScrollPanel {
        List<ClientPrincipal> selectedPrincipals = new ArrayList<ClientPrincipal>();
        UserTable container;

        public UsersListWidget() {
            super();
            this.addStyleName("resultsList");
        }

        public void refreshUsers(List<ClientPrincipal> principals) {
            this.clear();
            container = new UserTable(principals);
            CommonUtils.addAutomationId(container, "gwtcommon.client.addUsersAndGroupsPopup.userTable_0");
            DOM.setStyleAttribute(container.getElement(),"opacity", "0.0");
            this.add(container);
            FadeAnimation fader = new FadeAnimation(container.getElement());
            fader.fadeTo(1.0, 200);
        }

        public List<ClientPrincipal> getSelectedPrincipals() {
            return selectedPrincipals;
        }

        private class UserTable extends Grid {
            private List<ClientPrincipal> principals;

            public UserTable(List<ClientPrincipal> principals) {
                super(principals.size(), 3);
                this.principals = principals;
                if (CommonUtils.isInternetExplorer7()) {
                    this.setWidth("96%");
                }
                else {
                    this.setWidth("100%"); 
                }
                build();
            }

            private void build() {
                int i = 0;
                for(final ClientPrincipal principal : principals) {         
                    String icon = null;
                    String displayName = null;

                    switch(principal.getType()) {
                        case USER : {
                            icon = CommonUtils.COMMON_IMAGES.userIcon().getHTML();
                            displayName = principal.getFullName()!= null? principal.getFullName() + " (" + principal.getName() + ")":principal.getName();
                            break;
                        }
                        case GROUP: {
                            icon = CommonUtils.COMMON_IMAGES.groupIcon().getHTML();
                            displayName = "<b>" + principal.getName() + "</b>";
                            break;
                        }
                    }
                    HTML htmlIcon = new HTML(icon);
                    setWidget(i, 0, htmlIcon);
                    HTML htmlName = new HTML(displayName);
                    setWidget(i, 1, htmlName);

                    CheckBox select = new CheckBox();
                    if(selectedPrincipals.contains(principal)){
                        select.setChecked(true);
                    }
                    select.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                        }
                    });
                    
                    htmlIcon.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                            build(); //need to rebuild to check the box
                        }
                    });

                    htmlName.addClickListener(new ClickListener(){
                        public void onClick(Widget sender) {
                            // Add the user to the selected list so we know what all to add when
                            // the user clicks the submit button
                            if(!selectedPrincipals.contains(principal)){
                                selectedPrincipals.add(principal);
                            }
                            else {
                                selectedPrincipals.remove(principal);
                            }
                            build(); //need to rebuild to check the box
                        }
                    });
                    setWidget(i, 2, select);
                    this.getCellFormatter().setHorizontalAlignment(i,2, HasHorizontalAlignment.ALIGN_RIGHT);
                    
                    if(principals.size()==1) {
                        select.setChecked(true);
                        selectedPrincipals.add(principal);
                    }
                    ++i;
                }
            }
        }
    }


}
