/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author jsugg
 *
 */
public class CollapsiblePanel extends FlowPanel {

    private String label = "";
    private Widget widget = null;
    
    public CollapsiblePanel(String label, Widget widget) {
        super();
        this.label = label;
        this.widget = widget;
        rebuild();
    }

    public CollapsiblePanel(String label, String text) {
        super();
        this.label = label;
        this.widget = new HTML(text);
        rebuild();
    }

    public void rebuild() {
        clear();
        
        FlowPanel headerContainer = new FlowPanel();
        CommonUtils.addAutomationId(headerContainer, "gwtcommon.client.collapsiblePanel.flowPanel_headCont");
        headerContainer.addStyleName("collapsibleHeader");
        add(headerContainer);
        final ChevronTogglePanel toggle = new ChevronTogglePanel(this.label);
        CommonUtils.addAutomationId(toggle, "gwtcommon.client.collapsiblePanel.chevronTPanel");
        toggle.addToggleListener(new IToggleListener() {
            public void onToggle(int state) {
                if (state == 0) {
                    widget.setVisible(false);
                } else {
                    widget.setVisible(true);
                }
            }
        });
        headerContainer.add(toggle);
        
        if (widget != null) {
            widget.setStyleName("collapsibleWidget");
            widget.setVisible(false);
            add(widget);
        }
    }
    
    public class TogglePanel extends ClickPanel {
        
        private List<Image> images = new ArrayList<Image>();
        private int current = 0;
        private List<IToggleListener> listeners = new ArrayList<IToggleListener>();
        private String toggleLabel = null;
        
        public TogglePanel() {
            super();
            init();
        }
        
        public TogglePanel(String label) {
            super();
            this.toggleLabel = label;
            init();
        }
        
        private void init() {
            addClickListener(new ClickListener() {
                public void onClick(Widget sender) {
                    advance();
                }
            });            
        }
        
        public void addImage(Image image) {
            images.add(image);
            image.unsinkEvents(Event.ONCLICK);

            // if this is the first image then display it
            if (images.size() == 1) {
                refresh();
            }
        }
        
        public void advance() {
            advance(1);
        }
        
        public void advance(int i) {
            // bounds check
            if (images.size() == 0)
                return;
            
            // calculate the new image index -- use the modulo to wrap-around
            current = (current + i) % images.size();
            
            refresh();

            // notify anybody who might care
            fireListeners();
        }
        
        public void refresh() {
            // now set the display to show the right image
            clear();
            clearElements();
            Image image = images.get(current);
            image.setStyleName("collapsibleImage");
            DOM.appendChild(getElement(), image.getElement());
            if (this.toggleLabel != null) {
                Element anchor = DOM.createAnchor();
                DOM.setElementAttribute(anchor, "href", "##");
                DOM.setInnerHTML(anchor, this.toggleLabel);
                DOM.appendChild(getElement(), anchor);
            }
        }
        
        public void setToggleLabel(String label) {
            this.toggleLabel = label;
            refresh();
        }
        
        public void addToggleListener(IToggleListener listener) {
            if (!listeners.contains(listener))
                listeners.add(listener);
        }
        
        public void removeToggleListener(IToggleListener listener) {
            if (listeners.contains(listener))
                listeners.remove(listener);
        }
        
        protected void fireListeners() {
            for (int i = 0; i < listeners.size(); i++) {
                IToggleListener listener = listeners.get(i);
                listener.onToggle(current);
            }
        }
        
        protected void clearElements() {
            while (DOM.getChildCount(getElement()) > 0) {
                DOM.removeChild(getElement(), DOM.getChild(getElement(), 0));
            }
        }
    }
    
    public interface IToggleListener {
        public void onToggle(int state);
    }
    
    public class ChevronTogglePanel extends TogglePanel {

        public ChevronTogglePanel(String label) {
            super(label);
            addImage(new Image("images/style/chevron_right.png"));
            addImage(new Image("images/style/chevron_down.png"));
        }
    }
    
}
