package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.HashMap;
import java.util.Map;

import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.ui.UIObject;

public abstract class CommonUtils{
    private static CommonMessages messages = null;
    public static final String COMMON_IMAGES_PATH = "com/lombardisoftware/gwtcommon/public/images/";
    public static final String COMMON_ICONS_PATH = COMMON_IMAGES_PATH + "icons/";
    public static final String COMMON_STYLE_IMAGES_PATH = COMMON_IMAGES_PATH + "style/";
    public static final CommonImageBundle COMMON_IMAGES = GWT.create(CommonImageBundle.class);
    public static final CommonTreeImageBundle COMMON_TREE_IMAGES = GWT.create(CommonTreeImageBundle.class);
    public static final int TRANSITION_ANIMATION_INTERVAL = 200;

    /**
     * A sequence of alphanumerical characters, also including the underscore ("_") and dollar sign ("$"),
     * that cannot start with a number
     * ^[A-Za-z_\$] - Captures the first character and verifies that it is either a letter or "_" or "$"
     * [A-Za-z0-9_\$]* - Captures everything after the first letter and verfies it is alphanumeric or "_" or "$"
     */
    private final static String jsIdentifierRegex = "^[A-Za-z_\\$][A-Za-z0-9_\\$]*";

    /**
     * Holds all javascript reserved words
     */
    private static final Map<String, String> jsReservedWords = new HashMap<String, String>();

    static {
        // Intialize the javascript reserved words
        String[] words = new String[]{"abstract", "as","boolean","break","byte","case","catch","char","class",
                "continue","const","debugger","default","delete","do","double","else","enum","export","extends","false",
                "final","finally","float","for","function","goto","if","implements","import","in","instanceof","int",
                "interface","is","long","namespace","native","new","null","package","private","protected","public",
                "return","short","static","super","switch","synchronized","this","throw","throws","transient","true",
                "try","typeof","use","var","void","volatile","while","with"};

        // Put them into the map
        for(String word: words) {
            jsReservedWords.put(word,word);
        }
    }

    // It's not clear whether we need to distinguish these from
    // other calls to setElementId, or to use a different mechanism
    // for setting those ids.  I'm leaving the duplication in place
    // pending further review.
    public static void addAutomationId(UIObject o, String id) {
        addAutomationId( o.getElement(), id) ; 
    }
       
    public static void addAutomationId(Element e, String id) {
        DOM.setElementAttribute(e, "id", id) ;
    }
    
    /**
     * gwt java.util package doesn't support subList(!) so we roll our own
     *
     * Example:
     *  Consider List<String> X
     *  ---
     *    0   1   2   3
     *  | w | x | y | z |
     *  ---
     *  subList(X,1,4) returns x,y,z
     *  subList(X,2,3) returns y
     *  subList(X,3,3) return empty list
     *  subList(X,0,X.size()) returns w,x,y,z
     *
     * @param list to sublist
     * @param fromIndex low endpoint (inclusive) of the subList.
     * @param toIndex high endpoint (exclusive) of the subList.
    */
    public static <T> List<T> subList(List<T> list, int fromIndex, int toIndex) {
        if (fromIndex < 0) {
            throw new IndexOutOfBoundsException("fromIndex = " + fromIndex);
        }
        if (toIndex > list.size()) {
            throw new IndexOutOfBoundsException("toIndex = " + toIndex);
        }
        if (fromIndex > toIndex) {
            throw new IllegalArgumentException("fromIndex(" + fromIndex +") > toIndex(" + toIndex + ")");
        }

        // upper bound - lower bound then add one because we want the size of the list
        int subListSize = toIndex - fromIndex + 1;
        List<T> result = new ArrayList<T>(subListSize);
        for (int i = fromIndex; i < toIndex && i < list.size(); i++) {
            result.add(list.get(i));
        }
        return result;
    }

    public static CommonMessages getCommonMessages() {
        if (messages  == null) {
            messages = (CommonMessages) GWT.create(CommonMessages.class);
        }
        return messages;
    }

    
    public static void displayHelpURL(String url) {
        if (isRunningInAE()) {
            StringBuilder u = new StringBuilder(getBaseURL());
            u.append(getCommonMessages().helpPath()).append(url);
            displayHelpURLNative(u.toString());
        } else {
            Window.open(url, getCommonMessages().externalWindowTitle(), "");
        }
    }

    public static native boolean displayHelpURLNative(String url)
    /*-{
        try{
            netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
            var oMyComponent = Components.classes['@com.lombardi/AECom;1'].createInstance(Components.interfaces.IAECom);
            if(oMyComponent == null) {
                alert("XPCOM Component is null");
            }
            oMyComponent.displayHelpURL(url);
        } catch(e) {
            alert(e);
        }
    }-*/;

    /**
     * Test to see if we can get the the XPCOM priveleges and connect
     * @return
     */
    public static native boolean isRunningInAE()
    /*-{
        try{
            netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
            var oMyComponent = Components.classes['@com.lombardi/AECom;1'].createInstance(Components.interfaces.IAECom);
            if(oMyComponent == null)
            {
                return false;
            }

            return true;
            
        } catch(e) {
            return false;
        }
    }-*/;

    public static <T> List<T> singletonList(T element) {
        // sure wish GWT would let us use Collections.singletonList()...
        List<T> list = new ArrayList<T>();
        list.add(element);
        return list;
    }

    public static void setTop(final UIObject uiObject, final int top) {
        uiObject.getElement().getStyle().setPropertyPx("top", top);
    }

    public enum StylePosition { Normal, Absolute, Relative, Fixed }

    public static void setPosition(final UIObject uiObject, final StylePosition stylePosition) {
        uiObject.getElement().getStyle().setProperty("position", stylePosition.name().toLowerCase());
    }

    /**
     * Validate that the input string is a valid jsIdentifier.
     * @return
     */
    public static boolean isJSIdentifier(String input) {
        // Is it a javascript reserved word?
        if(isJSReservedWord(input)) {
            return false;
        }

        // Does it follow the rules of a javascript identifier?
        return input.matches(jsIdentifierRegex);

    }

    /**
     * Check if the input is a Javascript reserved word.
     *
     * @param input
     * @return
     */
    public static boolean isJSReservedWord(String input) {
        return jsReservedWords.containsValue(input);
    }

	/**
     * Returns the date in month/day/year format
     */
    public static String formatDate(long d) {
        Date date = new Date(d);
        return DateTimeFormat.getShortDateFormat().format(date);
    }

    /**
     * @param armyTime - wether to show 24hr time or 12 hr time
     * @param time - representation of a date as a long
     * @return - Formatted time string
     */
    public static String formatTime(long time, boolean armyTime) {
        Date date = new Date(time);

        StringBuffer buff = new StringBuffer();
        String AM_PM = "am";
        int hour = date.getHours();
        if ( !armyTime && hour > 12) {
            hour -= 12;
            AM_PM = "pm";
        }
        buff.append(hour);
        buff.append(":");
        int minutes = date.getMinutes();
        if(minutes<10) {
            buff.append("0");
        }
        buff.append(minutes);
                
        if (!armyTime) {
            buff.append(AM_PM);    
        }

        return buff.toString();
	}
    
    public static native String getUserAgent() 
    /*-{
        return navigator.userAgent.toLowerCase();
    }-*/;
       
    public static boolean isInternetExplorer7() {
        String agent = getUserAgent();
        return agent.contains("msie 7.0");
    }

    public static boolean isIE() {
        String agent = getUserAgent();
        return agent.contains("msie");
    }

    public static native String getLoginUser()
    /*-{
        try{
            netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
            var oMyComponent = Components.classes['@com.lombardi/AECom;1'].createInstance(Components.interfaces.IAECom);
            if(oMyComponent == null)
            {
                return null;
            }
            return oMyComponent.getLoginUser();
        } catch(e) {
            return null;
        }
    }-*/;
    
    public static native String getLoginPassword()
    /*-{
        try{
            netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
            var oMyComponent = Components.classes['@com.lombardi/AECom;1'].createInstance(Components.interfaces.IAECom);
            if(oMyComponent == null)
            {
                return null;
            }

            return oMyComponent.getLoginPassword();
        } catch(e) {
            return null;
        }
    }-*/;

    /**
     * Invalidates current user session and displays login page.
     * It epires session cookies and forwards to context root. This causes user to be
     * redirected to login page.
     */
    public static void invalidateSession() {
        Cookies.setCookie("JSESSIONID", "", new Date(0L));
        Cookies.setCookie("JSESSIONIDSSO", "", new Date(0L));
        Window.Location.assign(getContextPath());
    }

    /**
     * Returns application context path.
     * Context starts from slash and does NOT include trailing slash.
     *
     * The method extracts path as first part of page URL following by host name and port.
     * For URL "http://hostname:12345/context/someUrl/someMore" we get "/context".
     *
     * @return String context
     */
    public static String getContextPath() {
        String urlPath = Window.Location.getPath();
        // URL path starts from a slash so searching for slash starting from 2nd char.
        int contextEndIndex = urlPath.indexOf("/", 1);
        String context = urlPath.substring(0, contextEndIndex);
        return context;
    }
    
    /**
     * @return transport + hostname + port of the URL of this application
     */
    public static String getBaseURL() {
        // start from the moduleURL
        String moduleURL = GWT.getModuleBaseURL();
        
        // first find the "//" -- the plus 2 accounts for the length of the "//"
        int end = moduleURL.indexOf("//") + 2;
        end = moduleURL.indexOf('/', end);
        String result = moduleURL.substring(0, end);
        return result;
    }
    
}

