/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author bcampbell
 * @date: Jun 23, 2008
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.ClickListenerCollection;
import com.google.gwt.user.client.ui.SourcesTableEvents;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.TableListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.SourcesClickEvents;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Label;

/**
 * <p>
 * A LibraryButton wraps a 3 X 1 grid so that the user can specify left, middle, and right images.  In this
 * case to do rounded corners and gradient background.
 * </p>
 * <h3>CSS Style Rules</h3>
 * <ul class='css'>
 * <li>.libraryButton { the main button class }</li>
 * <li>.libraryButton .left { the left-most cell }</li>
 * <li>.libraryButton .middle { the middle cell }</li>
 * <li>.libraryButton .text { div for the button text }</li>
 * <li>.libraryButton .right { the right-most cell }</li>
 * </ul>
 */
public class LibraryButton extends SimplePanel implements HasText, SourcesClickEvents {

    private ClickListenerCollection clickListener = new ClickListenerCollection();
    private Label textDiv;
    private Grid content;
    
    public LibraryButton() {
        init(false);
    }

    public LibraryButton(String text) {        
        init(false);
        textDiv.setText(text);        
    }
    
    public LibraryButton(String text , boolean disable) {        
        init(disable);
        textDiv.setText(text);        
	}    
    
    public void enable(boolean enable) {
        textDiv.setStyleName(enable?"buttonText":"buttonTextDisable");
    }
    
    private void init(boolean disable) {
        setStyleName("libraryButton");
        
        // Setup grid size and set style names
        content = new Grid(1,3);

        // Zero cell spacing.
        content.getElement().getStyle().setProperty("borderCollapse", "collapse");
        content.setCellSpacing(0); // IE7 does not override cell-spacing with border-collapse.
        content.setCellPadding(0); // IE6 does not override cell-padding with border-collapse.
        
        content.getCellFormatter().addStyleName(0, 0, "buttonLeft");
        content.getCellFormatter().addStyleName(0, 1, "middle");
        content.getCellFormatter().addStyleName(0, 2, "buttonRight");

        // Add the button label area
        textDiv = new Label();
        textDiv.setStyleName(disable?"buttonTextDisable":"buttonText");
        content.setWidget(0, 1, textDiv);
        content.addTableListener(new TableListener() {
            public void onCellClicked(SourcesTableEvents sender, int row, int cell) {
                clickListener.fireClick(LibraryButton.this);
            }
        });
        this.add(content);
    }
    
    public String getText() {
		return textDiv.getText();
	}

	public void setText(String text) {
        textDiv.setText(text);
    }

	public void addClickListener(ClickListener listener) {
		clickListener.add(listener);
	}

	public void removeClickListener(ClickListener listener) {
		clickListener.remove(listener);
	}

}

