/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.ComplexPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.LIElement;

/**
 * The class is complex container widget that wraps its elements into unordered list.
 * 
 * @author Slava Nadvorny vnadvorny
 */
public class ListPanel extends ComplexPanel {

    /**
     * Creates an empty list panel.
     */
    public ListPanel() {
        setElement(Document.get().createULElement());
    }

    /**
     * Adds a new child widget to the panel.
     *
     * @param widget the widget to be added
     */
    @Override
    public void add(Widget widget) {
        LIElement li = Document.get().createLIElement();
        
        li.appendChild(widget.getElement());
        
        // Detach new child.
        widget.removeFromParent();

        // Logical attach.
        getChildren().add(widget);

        // Physical attach.
        DOM.appendChild(getElement(), li.<Element>cast());

        // Adopt.
        adopt(widget);
    }

    @Override
    public void onBrowserEvent(Event event) {
        if (event.getTypeInt() == Event.ONCLICK) {
            for (Widget widget : getChildren()) {
                if (event.getTarget().equals(widget.getElement())) {
                    widget.onBrowserEvent(event);
                }
            }
        } 
        super.onBrowserEvent(event);    //To change body of overridden methods use File | Settings | File Templates.
    }
}
