package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.FormHandler;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.FormSubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormSubmitEvent;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class LoginDialog extends DialogBox {

    private static LoginDialog instance = null;
    
    private List<Command> commandQueue = new ArrayList<Command>();
    
    private HTML header = null;
    private TextBox userText = null;
    private PasswordTextBox pwText = null;

    private boolean showing = false;
    
    public static LoginDialog getInstance(String message, Command command) {
        if (instance == null)
            instance = new LoginDialog();
        
        instance.commandQueue.add(command);
        instance.initialize("Please enter your user name and password to log in.");
        return instance;
    }
    
    private LoginDialog() {
        build();
    }

    private void initialize(String message) {
        if (showing)
            return;
        
        header.setHTML(message + "<br><br>");
        userText.setText("");
        pwText.setText("");
        userText.setFocus(true);
    }
    
    @Override
    public void show() {
        if (showing)
            return;
        
        super.show();
    }
    
    @Override
    public void center() {
        if (showing)
            return;
        
        super.center();
    }

    @Override
    public void hide() {
        super.hide();
        showing = false;
    }
    
    protected void build() {
        this.clear();
        this.addStyleName("loginPopup");
        
        DOM.setStyleAttribute(getElement(), "display", "block");

        FlowPanel container = new FlowPanel();
        container.setStyleName("login-dialog");
        setWidget(container);

        this.setText("Login");
        
        FlowPanel contents = new FlowPanel();
        container.add(contents);
        header = new HTML();
        contents.add(header);
        
        final FormPanel form = new FormPanel();
        form.setMethod(FormPanel.METHOD_POST);
        form.setAction(GWT.getHostPageBaseURL() + "/repository/j_security_check");
        contents.add(form);
        VerticalPanel formContents = new VerticalPanel();
        form.setWidget(formContents);
        
        // now add the inputs
        HorizontalPanel userPanel = new HorizontalPanel();
        formContents.add(userPanel);
        Label userLabel = new Label("User Name:");
        userLabel.addStyleName("loginPopupNameLabel");
        userPanel.add(userLabel);
        userText = new TextBox();
        userPanel.add(userText);
        userText.setName("j_username");

        HTML spacer = new HTML("<br>");
        formContents.add(spacer);
        
        HorizontalPanel pwPanel = new HorizontalPanel();
        formContents.add(pwPanel);
        Label pwLabel = new Label("Password:");
        pwLabel.addStyleName("loginPopupPassLabel");
        pwPanel.add(pwLabel);
        pwText = new PasswordTextBox();
        pwText.addStyleName("loginPopupPassBox");
        pwPanel.add(pwText);
        pwText.setName("j_password");
        pwText.setWidth("157px");
        
        // and a submit button
        Button submit = new Button("Login");
        submit.addStyleName("loginPopupSubmit");
        formContents.add(submit);
        
        submit.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                form.submit();
            }
        });

        form.addFormHandler(new FormHandler() {
            public void onSubmit(FormSubmitEvent event) {
            }
            public void onSubmitComplete(FormSubmitCompleteEvent event) {
                executeQueue();
                LoginDialog.this.hide();
            }
        });
    }

    protected void executeQueue() {
        for (Command command: commandQueue) {
            command.execute();
        }
    }   
}
