/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import java.util.ArrayList;
import java.util.EventListener;

import com.google.gwt.user.client.ui.Widget;

/**
 * The class controls paging.
 * 
 * @author Slava Nadvorny
 */
public class Paginator {
    
    public static final int DEFAULT_PAGE_SIZE = 20;

    private int pageSize = DEFAULT_PAGE_SIZE;
    private int currentPage = 1;
    private int pageCount = 1;
    private int elementCount = -1;
    private ArrayList<PageListener> listeners;
    private PaginatorWidget widget;

    public Paginator() {
        widget = new PaginatorWidget(this);
    }
    
    public Widget getWidget() {
        return widget;
    }

    public void setPage(int newPageNumber) {
        if (newPageNumber != currentPage) {
            currentPage = newPageNumber;
            firePageListeners(newPageNumber);
            widget.refreshWidget(pageCount, currentPage);
        }
    }


    public int getFirstElementOnCurrentPage() {
        return pageSize * (currentPage - 1);
    }

    public int getElementCount() {
        return elementCount;
    }
    
    public void setPaging(int newPageCount, int newCurrentPage) {
        currentPage = newCurrentPage;
        pageCount = newPageCount;
        recalculatePaging();
    }

    private void recalculatePaging() {
        if (this.currentPage > pageCount) {
            this.currentPage = 1;
        }
        widget.refreshWidget(pageCount, currentPage);
    }

    public void setPageCount(int newPageCount) {
        pageCount = newPageCount;
        recalculatePaging();
    }
    
    public void setPageSize(int newPageSize) {
        pageSize = newPageSize;
        recalculatePaging();
    }
    
    public int getPageSize() {
        return pageSize;
    }
    
    public void setElementCount(int newElementCount) {
        if (this.elementCount != newElementCount) {
            this.elementCount = newElementCount;
            int newPageCount = (int) Math.ceil((double) newElementCount / pageSize);
            setPageCount(newPageCount);
        }
    }

    private void firePageListeners(int pageNumber) {
        if (listeners != null) {
            for (PageListener listener : listeners) {
                listener.firePageChanged(pageNumber);
            }
        }
    }

    public void addPageListener(PageListener listener) {
        if (listeners == null) {
            listeners = new ArrayList<PageListener>();
        }
        listeners.add(listener);
    }

    public void removePageListener(PageListener listener) {
        if (listeners != null) {
            listeners.remove(listener);
        }
    }

    public int getPageCount() {
        return pageCount;
    }

    public static interface PageListener extends EventListener {
        public void firePageChanged(int newPage);
    }
}


