/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.Event;

/**
 * The widget displays page selection. It works with {@link Paginator} as controller. 
 * 
 * @author Slava Nadvorny
 */
public class PaginatorWidget extends Widget {

    private ListPanel panel;
    private SimplePanel parent;
    private Paginator paginator;
    private FlowPanel root;

    public PaginatorWidget(Paginator paginator) {
        root = new FlowPanel();
        root.addStyleName("pagingWidget");
        root.setVisible(false);

        parent = new SimplePanel();
        parent.addStyleName("pagingParent");
        root.add(parent);

        panel = new ListPanel();
        panel.setStyleName("paging");
        parent.add(panel);
        
        setElement(root.getElement());
        
        this.paginator = paginator;
    }

    @Override
    public void onBrowserEvent(Event event) {
        if (event.getTypeInt() == Event.ONCLICK) {
            panel.onBrowserEvent(event);
        }
        super.onBrowserEvent(event);
    }

    /**
     * Rebuilds paging widget.
     * 
     * @param pageCount Number of pages.
     * @param currentPage Page to mark as current.
     */
    public void refreshWidget(int pageCount, int currentPage) {
        assert pageCount >= currentPage : "Current page number should be less or equal to page count.";
        panel.clear();
        // Show control only if there are more than one page.
        if (pageCount > 1) {
            if (currentPage > 2) {
                HTML newest = getPageLink(1, "&laquo; First");
                panel.add(newest);
            }
            if (currentPage > 1) {
                HTML newer = getPageLink(currentPage - 1, "&#8249; Back");
                panel.add(newer);
            }

            int rangeStart = paginator.getFirstElementOnCurrentPage() + 1;
            int expectedEnd = rangeStart + paginator.getPageSize() - 1;
            int rangeEnd = (expectedEnd > paginator.getElementCount()) ? paginator.getElementCount() : expectedEnd;

            Label viewing = new HTML(rangeStart + " - " + rangeEnd + " of " + paginator.getElementCount());
            viewing.addStyleName("viewing");
            panel.add(viewing);

            int lastPage = paginator.getPageCount();

            if (currentPage < lastPage) {
                HTML older = getPageLink(currentPage + 1, "Next &#8250;");
                panel.add(older);
            }
            if (currentPage < lastPage - 1) {
                HTML oldest = getPageLink(lastPage, "Last &raquo;");
                panel.add(oldest);
            }
        
            root.setVisible(true);
        } else {
            root.setVisible(false);
        }
    }

    private HTML getPageLink(int pageNumber, String text) {
        HTML link = new HTML(text);
        link.addClickListener(new PageClickListener(pageNumber));
        link.addStyleName("pageLink");
        return link;
    }

    private class PageClickListener implements ClickListener {
        private int pageNumber;

        public PageClickListener(int pageNumber) {
            this.pageNumber = pageNumber;
        }

        public void onClick(Widget sender) {
            paginator.setPage(pageNumber);
        }
    }

}
