/*
 * Copyright (C) 2009 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client;

import java.util.List;

import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.DecoratorPanel;

/**
 * The horizontal menu with green buttons like in Process App details (Snapshot) (History) (Manage).
 *
 * @author jsugg
 *
 */
public class PipesMenu extends FlowPanel {
    private static final String SELECTED_SUBSTYLE = "Selected";

    List<ActionLink> actions;
    private boolean isCurrentClickable = false;

    public PipesMenu(List<ActionLink> actions) {
        this.actions = actions;

        setStyleName("proPipes");
        refreshActions();
    }

    protected void refreshActions() {
        clear();
        int numActions = actions.size();
        for (int i = 0; i < numActions; i++) {
            ActionLink action = this.actions.get(i);
            DecoratorPanel actionPanel = new DecoratorPanel();
            actionPanel.setStyleName("twChoiceButton");
            if(isCurrentClickable) {
                actionPanel.addStyleDependentName("Clickable");
            } else {
                actionPanel.removeStyleDependentName("Clickable");
            }

            HTML actionLink = new HTML(action.name);
            CommonUtils.addAutomationId(actionLink, "gwtcommon.client.pipesMenu.link_0");
            if((!action.selected || isCurrentClickable) && action.listener != null) {
                actionLink.addClickListener(action.listener);
            }

            if(action.selected) {
                actionPanel.addStyleDependentName(SELECTED_SUBSTYLE);
            } else {
                actionPanel.removeStyleDependentName(SELECTED_SUBSTYLE);
            }

            actionPanel.add(actionLink);

            add(actionPanel);
        }
    }

    public boolean isCurrentClickable() {
        return isCurrentClickable;
    }

    public void setCurrentClickable(boolean isCurrentClickable) {
        if(this.isCurrentClickable != isCurrentClickable) {
            this.isCurrentClickable = isCurrentClickable;

            refreshActions();
        }
    }

    public void select(int index) {
        int numActions = actions.size();
        if(index < 0 || index >= numActions) {
            throw new IndexOutOfBoundsException("Invalid index while selecting pipe");
        }

        for(int i = 0; i < numActions; i++) {
            actions.get(i).selected = index == i;
        }

        refreshActions();
    }

    /**
     * Selects a link in the Pipes Menu, deselects all if the link is not in the list.
     *
     * @param linkToSelect ActionLink to mark as selected.
     */
    public void select(ActionLink linkToSelect) {

        for (ActionLink action : actions) {

            if (action.equals(linkToSelect)) {
                action.select();
            } else {
                action.unselect();
            }
        }
        
        refreshActions();
    }
    
    public static class ActionLink {
        public String name = "";
        public ClickListener listener = null;
        public boolean selected = false;
        public Image icon = null;
        
        public ActionLink(String name, ClickListener listener) {
            this.name = name;
            this.listener = listener;
        }

        public ActionLink(String name, Image icon, ClickListener listener) {
            this.name = name;
            this.listener = listener;
            this.icon = icon;
        }

        /**
         * Selects the acton link.
         */
        public void select() {
            selected = true;
        }

        /**
         * Deselects the action link.
         */
        public void unselect() {
            selected = false;
        }
    }
}
