/*
 * Copyright (C) 2007 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.ClickListenerCollection;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.Label;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;

/**
 * @author Bryan Campbell
 */
public class PrincipalRow extends FlowPanel {

    private ClickListenerCollection clickListeners;
    private ClientPrincipal principal;

    /**
     * Creates a new ProcessRow for the given process.
     */
    public PrincipalRow(ClientPrincipal principal, boolean includeRemoveButton) {
        super();
        setStyleName("userAccessRow");
        sinkEvents(Event.ONCLICK);
        sinkEvents(Event.MOUSEEVENTS);

        this.principal = principal;

        Image principalIcon = principal.isUser() ? CommonUtils.COMMON_IMAGES.userIcon().createImage() : CommonUtils.COMMON_IMAGES.groupIcon().createImage();
        principalIcon.setStyleName("principalRowIcon");
        this.add(principalIcon);

        Span principalName = new Span(principal.getListDisplayName());
        principalName.setStyleName("principalRowName");
        if(!principal.isUser()) {
            DOM.setStyleAttribute(principalName.getElement(), "font-weight", "bold");
        }
        this.add(principalName);

        if (includeRemoveButton && !principal.isReadOnly()) {
            // Create the Trash button to remove a user from a project
            Image removeIcon = CommonUtils.COMMON_IMAGES.removeUserIcon().createImage();
            removeIcon.setStyleName("removeButton");
            removeIcon.addClickListener(new ClickListener() {
                public void onClick(Widget sender) {
                    fadeAndSlideUpAnimation();
                    doRemoveRow();
                }
            });

            // Add the trash icon to the panel
            this.add(removeIcon);
        }
    }

    public ClientPrincipal getPrincipal() {
        return principal;
    }

    public void addClickListener(ClickListener listener) {
        if (clickListeners == null) {
            clickListeners = new ClickListenerCollection();
        }
        clickListeners.add(listener);
    }

    @Override
    public void onBrowserEvent(Event event) {
        switch (DOM.eventGetType(event)) {
            case Event.ONCLICK:
                onClick();
                break;
            case Event.ONMOUSEOUT:
                onMouseOut();
                break;
            case Event.ONMOUSEOVER:
                onMouseOver();
                break;

        }
        super.onBrowserEvent(event);
    }

    private void onClick() {
        if (clickListeners != null) {
            clickListeners.fireClick(this);
        }
    }

    private void onMouseOver() {
        addStyleName("hover");
    }

    private void onMouseOut() {
        removeStyleName("hover");
    }
    

    /**
     * Animation to fade the row out, then slide up (should be fixed later though)
     */
    public void fadeAndSlideUpAnimation() {
        FadeAnimation fader = new FadeAnimation(this);
        fader.fadeTo(0.0, 1000, new Command() {
            public void execute() {
                PrincipalRow.this.removeFromParent();
            }
        });
    }

    protected void doRemoveRow() {
    }

    public void select() {
        setStyleName("userAccessRowSelected");
    }

    public void unselect() {
        setStyleName("userAccessRow");
    }

    public static class Header extends FlowPanel {
        public Header() {
            setStyleName("tableHeader");

            Label removeHeaderLabel = new Label("Remove");
            removeHeaderLabel.setStyleName("removeColumnHeader");

            add(removeHeaderLabel);
        }
    }

}
