/**
 * Copyright (C) 2006 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * @author bcampbell
 * @date: Oct 20, 2008
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.lombardisoftware.gwtcommon.client.clientObjects.ClientPrincipal;
import com.lombardisoftware.gwtcommon.client.LibraryButton;
import com.lombardisoftware.gwtcommon.client.AbstractRepositoryPopup;

import java.util.List;

public class PrincipalsWidget extends FlowPanel {

    private boolean includeAddButton;
    private boolean includeRemoveButtons;
    private boolean supportsSelection;
    private FlowPanel rowContainer;
    private PrincipalRow selectedRow;
    protected AsyncCallback<List<ClientPrincipal>> refreshMembersCallback = new AsyncCallback<List<ClientPrincipal>>() {
        public void onFailure(Throwable caught) {};
        public void onSuccess(List<ClientPrincipal> result) {
            refresh(result);
        }
    };

    public PrincipalsWidget(boolean includeAddButton, boolean includeRemoveButtons, boolean supportsSelection) {
        this.includeAddButton = includeAddButton;
        this.includeRemoveButtons = includeRemoveButtons;
        this.supportsSelection = supportsSelection;
        
        // Widget representing the user management list widget
        setStyleName("manageUserAccessWidget");

        // Add the header for UserAccessRow
        Widget header = createHeader();
        if (header != null) {
            add(header);
        }
        
        // Widget to hold all the userAccessRows
        rowContainer = new FlowPanel();
        rowContainer.setStyleName("userAccessRowsContainer");
        
        // Add the row container to the management widget
        add(rowContainer);
        
        if (includeAddButton) {    
            // Add the add users button
            LibraryButton addButton = new LibraryButton("Add Users/Groups");
            add(addButton);
            addButton.addClickListener(createAddListener());
        }
    }
    
    public ClickListener createAddListener() {
        return new ClickListener() {
            public void onClick(Widget sender) {
                // Create the popup dialog to add members
                AddUsersAndGroupsPopup popup = createPopup(PrincipalsWidget.this);
                if (popup != null) {
                    popup.addCloseListener(new AbstractRepositoryPopup.ICloseListener() {
                        public void onClose(AbstractRepositoryPopup theClosed) {
                            if(!theClosed.isCancelled()) {
                                internalRefresh();
                            }
                        }
                    });
                    popup.showCentered();
                }
            }
        };
    }
    
    public boolean isIncludeAddButton() {
        return includeAddButton;
    }

    public boolean isIncludeRemoveButtons() {
        return includeRemoveButtons;
    }
    
    public boolean isSupportsSelection() {
        return supportsSelection;
    }

    public void addPrincipals(List<ClientPrincipal> principals) {
        if (principals != null) {
            for(ClientPrincipal principal: principals) {
                final PrincipalRow row = createRow(principal);
                if (supportsSelection) {
                    row.addClickListener(new ClickListener() {
                        public void onClick(Widget sender) {
                            if (selectedRow != null) {
                                selectedRow.unselect();                                
                            }
                            row.select();
                            selectedRow = row;
                        }                        
                    });
                }
                rowContainer.add(row);
            }
        }
    }
    
    public void refresh(List<ClientPrincipal> principals) {
        selectedRow = null;
        rowContainer.clear();
        addPrincipals(principals);
    }

    /**
     * The widgets should know how to get their principals
     */
    public void refresh() {
        internalRefresh();
    }

    protected AddUsersAndGroupsPopup createPopup(Widget parent) {
        return null;
    }
    
    protected void internalRefresh() {
    }

    public void setTableHeight(String height) {
        rowContainer.setHeight(height);
    }
    
    protected FlowPanel createHeader() {
        return new PrincipalRow.Header();
    }
    
    protected PrincipalRow createRow(ClientPrincipal principal) {
        return new PrincipalRow(principal, includeRemoveButtons);
    }
    

}
