/*
 * Copyright (C) 2008 Lombardi Software
 * http://www.lombardi.com/
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights or licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.lombardisoftware.gwtcommon.client;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.MouseListenerAdapter;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;

public class TooltipListener extends MouseListenerAdapter {

    private static Tooltip tooltip;
    private String text;
    private int offsetX = -30;
    private int offsetY = 20;

    private static class Tooltip extends PopupPanel {

        public Tooltip(Widget sender, int offsetX, int offsetY, String text) {
            super(true);
            getElement().setClassName("styledTooltipPopup");
            sinkEvents(Event.MOUSEEVENTS);
            DOM.setStyleAttribute(getElement(), "zIndex", "10000");
            setAnimationEnabled(false);
            final FlowPanel popUp = new FlowPanel();
            SimplePanel arrow = new SimplePanel();
            arrow.setStyleName("styledTooltipArrow");
            HTML content = new HTML(text);
            content.setStyleName("styledTooltip");
            popUp.add(arrow);
            popUp.add(content);
            add(popUp);
            int left = sender.getAbsoluteLeft() + offsetX;
            int top = sender.getAbsoluteTop() + offsetY;
            setPopupPosition(left, top);
        }

        @Override
        public void onBrowserEvent(Event event) {
            switch(event.getTypeInt()) {
                case Event.ONMOUSEOUT:
                    this.hide();
            }
        }
    }

    public TooltipListener(String text) {
        this.text = text;
    }

    public void setText(String text) {
        this.text = text;
    }

    @Override
    public void onMouseEnter(Widget sender) {
        if (tooltip != null) {
            tooltip.hide();
        }
        tooltip = new Tooltip(sender, offsetX, offsetY, text);
        tooltip.show();
    }

    /**
     * This will force current tooltip to hide.
     */
    public static void forceHide() {
        if (tooltip != null) {
            tooltip.hide();
        }
    }
}