/*
 * Copyright (C) 2009 Lombardi Software
 *  http://www.lombardi.com/
 *  All rights reserved.
 *
 * This software is the confidential and proprietary information of Lombardi
 * Software ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement and/or Non-Disclosure Agreement you entered into
 * with Lombardi Software. No rights are licenses are granted with respect to
 * the Confidential Information except as expressly set forth in such License
 * Agreement and/or Non-Disclosure Agreement.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL LOMBARDI
 * SOFTWARE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.lombardisoftware.gwtcommon.client.help;

import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.ClickListener;

import com.lombardisoftware.gwtcommon.client.CommonUtils;

/**
 * A factory for help system widgets.
 *
 * @author Slava Nadvorny vnadvorny
 */
public class HelpWidgetFactory {

    /**
     * The factory is pure static.
     */
    protected HelpWidgetFactory() {
    }

    /**
     * Creates new information widget to display inline help in Repository.
     *
     * @param provider Information provider to read information data from.
     * @return A widget to display help info in right panel of Repository.
     */
    public static Widget createInformationWidget(HelpTopicProvider provider) {
        return new HelpWidget(provider);
    }

    /**
     * Creates information link. This opens corresponding help topic when user
     * clicks on question mark in top right corner of Process Center (ex Repository).
     *
     * @param provider Help URL and information URL provider.
     * @return A widget to link to Question Mark or null if no information url given.
     */
    public static Widget createInformationLinkWidget(HelpTopicProvider provider) {

        final String informationURL = provider.getInformationURL();

        if (informationURL == null || informationURL.length() == 0) {
            // TODO: Should route to root help page.
            return null;
        }

        Hyperlink link = new Hyperlink(provider.getHelpLink(), "#help");
        CommonUtils.addAutomationId(link, "repository.client.adminPanel.hyperLink_0");

        link.addClickListener(new ClickListener() {

            public void onClick(Widget sender) {
                CommonUtils.displayHelpURL(informationURL);
            }
        });

        return link;
    }
}


