package com.lombardisoftware.gwtcommon.server;

import java.io.IOException;
import java.util.Calendar;
import javax.servlet.Filter;
import javax.servlet.FilterConfig;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

/**
 * The class modifies http headers to enable caching on files.
 *
 * @author Slava Nadvorny vnadvorny
 */
public class CacheHeaderFilter implements Filter {
    FilterConfig filterConfig;

    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain)
            throws IOException, ServletException {

        HttpServletResponse response = (HttpServletResponse) res;

        response.setHeader("Pragma", "");
        response.setHeader("Cache-Control", "max-age=" + 60 * 60 * 24 * 10);
        Calendar date = Calendar.getInstance();
        response.setDateHeader("Last-Modified", date.getTimeInMillis());
        date.add(Calendar.DAY_OF_YEAR, 10);
        response.setDateHeader("Expires", date.getTimeInMillis());

        // Chain the request/response.
        chain.doFilter(req, response);
    }

    public void init(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
    }

    public void destroy() {
        this.filterConfig = null;
    }
}